# -*- coding: ascii -*-
#
#  svgmanager.py - The Manager of SVG
#  Copyright (C) 2004 by Takuya KAWAHARA <num@sann.ne.jp>
#  Copyright (C) 2004 by Atzm WATANABE <sitosito@p.chan.ne.jp>
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License (version 2) as
#  published by the Free Software Foundation.  It is distributed in the
#  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE.  See the GNU General Public License for more details.
#
# $Id: svgmanager.py,v 1.43 2004/10/28 20:52:54 atzm Exp $

import gtk, gobject
import os, sys, shutil
import string, re

from showingsurface      import ShowingSurface
from common              import *
from viewer.viewercommon import *

class SVGManagerWindow(gtk.Window):
	def __init__(self, app, browse_command, ghostmanager):
		self.app = app
		self.browse_command = browse_command
		self.ghostmanager   = ghostmanager

		accel_group = gtk.AccelGroup()

## Create status (left) window
		## Create GhostList window

		self.ghost_list_str = gtk.ListStore(gobject.TYPE_BOOLEAN, gobject.TYPE_STRING,
									gobject.TYPE_STRING,  gobject.TYPE_STRING)

		self.ghost_list = gtk.TreeView(self.ghost_list_str)
		self.ghost_list.set_enable_search(gtk.TRUE)
		self.ghost_list.set_rules_hint(gtk.TRUE)
		self.ghost_list.set_search_column(gtk.TRUE)
		self.ghost_list.set_reorderable(gtk.TRUE)
		self.ghost_list.connect("cursor-changed", self.cursor_changed)
		self.ghost_list.connect("select-cursor-row", self.select_row)

		self.get_config()

		# make ghostlist column
		columns = [
			gtk.TreeViewColumn(unicode(_('Use'), 'utf-8')),
			gtk.TreeViewColumn(unicode(_('Ghost'), 'utf-8')),
			gtk.TreeViewColumn(unicode(_('FMO'), 'utf-8')),
			gtk.TreeViewColumn(unicode(_('Path'), 'utf-8')),
			]
		for i in range(len(columns)):
			columns[i].set_resizable(gtk.TRUE)
			columns[i].set_clickable(gtk.TRUE)
			columns[i].connect("clicked", self.sort, i)
			self.ghost_list.append_column(columns[i])

			if i == 0:
				cell = gtk.CellRendererToggle()
				cell.connect("toggled", self.check_use_toggle)
				columns[i].pack_start(cell, gtk.TRUE)
				columns[i].add_attribute(cell, 'active', i)
			else:
				cell = gtk.CellRendererText()
				columns[i].pack_start(cell, gtk.TRUE)
				columns[i].add_attribute(cell, 'text', i)
				if   i == 1: cell.set_property('width', 80)
				elif i == 2: cell.set_property('width', 40)
				#else:        cell.set_property('width', 140) # FIXME : kimeuchi ikunai

		self.sw = gtk.ScrolledWindow()
		self.sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.sw.set_shadow_type(gtk.SHADOW_IN)
		self.sw.add(self.ghost_list)

		# Create GhostInformation main
		self.sakura_box = gtk.Image()
		self.unyuu_box  = gtk.Image()

		sakura_f = gtk.Frame()
		sakura_f.set_border_width(0)
		sakura_f.set_size_request(SIZE_SURFACE_H + 1, SIZE_SURFACE_V + 1)
		sakura_f.set_shadow_type(gtk.SHADOW_NONE)
		sakura_f.add(self.sakura_box)

		unyuu_f = gtk.Frame()
		unyuu_f.set_border_width(0)
		unyuu_f.set_size_request(SIZE_SURFACE_H + 1, SIZE_SURFACE_V + 1)
		unyuu_f.set_shadow_type(gtk.SHADOW_NONE)
		unyuu_f.add(self.unyuu_box)

		#info_ghost is 'showing surface [0] and [10]'
		info_ghost = gtk.HBox(gtk.FALSE)
		info_ghost.pack_start(unyuu_f, gtk.TRUE, gtk.FALSE, 0)
		info_ghost.pack_start(sakura_f, gtk.TRUE, gtk.FALSE, 0)

		self.button_id = None
		self.url_button = get_icon_button(gtk.STOCK_JUMP_TO, size=gtk.ICON_SIZE_BUTTON,
										  relief=gtk.RELIEF_NORMAL, label=unicode(_('URL'), 'utf-8'))
		self.url_button.set_sensitive(gtk.FALSE)
		self.url_button.add_accelerator("clicked",  accel_group, ord('J'),
										gtk.gdk.CONTROL_MASK|gtk.gdk.MOD1_MASK,
										gtk.ACCEL_VISIBLE)

		self.button_id_s = None
		self.sur_button = get_icon_button(gtk.STOCK_INDEX, size=gtk.ICON_SIZE_BUTTON,
										  relief=gtk.RELIEF_NORMAL, label=unicode(_('Surface'), 'utf-8'))
		self.sur_button.set_sensitive(gtk.FALSE)
		self.sur_button.add_accelerator("clicked",  accel_group, ord('S'),
										gtk.gdk.CONTROL_MASK|gtk.gdk.MOD1_MASK,
										gtk.ACCEL_VISIBLE)

		buttonbox = gtk.HBox(gtk.TRUE)
		buttonbox.pack_start(self.url_button, gtk.FALSE, gtk.FALSE, 0)
		buttonbox.pack_start(self.sur_button, gtk.FALSE, gtk.FALSE, 0)

		info_ghost_a = gtk.VBox(gtk.FALSE)
		info_ghost_a.pack_start(info_ghost, gtk.TRUE, gtk.FALSE, 0)
		info_ghost_a.pack_start(buttonbox, gtk.FALSE, gtk.FALSE, 0)

		## Showing Ghost Status from svg.txt
		self.info_svg_str = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
		info_svg = gtk.TreeView(self.info_svg_str)
		info_svg.set_headers_visible(gtk.FALSE)
		info_svg.set_rules_hint(gtk.TRUE)

		svg_cols = [
			gtk.TreeViewColumn(),
			gtk.TreeViewColumn(unicode(_('Ghost Status'), 'utf-8'))
			]
		for i in range(len(svg_cols)):
			info_svg.append_column(svg_cols[i])
			cell = gtk.CellRendererText()
			svg_cols[i].pack_start(cell, gtk.TRUE)
			svg_cols[i].add_attribute(cell, 'text', i)

		isv = gtk.ScrolledWindow()
		isv.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		isv.set_size_request(200, 200)
		isv.set_shadow_type(gtk.SHADOW_IN)
		isv.add(info_svg)

		info_hbox = gtk.HBox()
		info_hbox.pack_start(info_ghost_a, gtk.FALSE, gtk.FALSE, 0)
		info_hbox.pack_start(isv, gtk.TRUE, gtk.TRUE, 0)

		v_paned = gtk.VPaned()
		v_paned.pack1(info_hbox)
		v_paned.pack2(self.sw)

		self.ghost_name = gtk.Entry()
		self.ghost_name.set_editable(gtk.FALSE)

		# Create Search Entry
		self.msg = unicode(_("Search"), "utf-8")
		self.search_entry = gtk.Entry()
		self.search_entry.set_text(self.msg)
		self.search_entry.connect("activate", self.search)
		self.search_entry.connect("key-press-event", self.entry_keypress)
		self.search_entry.connect("focus-in-event", self.set_entry_guide, 'in')
		self.search_entry.connect("focus-out-event", self.set_entry_guide, 'out')

		self.s_arrow = gtk.Arrow(gtk.ARROW_DOWN, gtk.SHADOW_NONE)

		search_allow_button = gtk.ToggleButton()
		search_allow_button.add(self.s_arrow)
		search_allow_button.connect('clicked', 
				lambda x: self.s_arrow.set((not self.s_arrow.get_property('arrow-type')),
				gtk.SHADOW_NONE))
		search_allow_button.connect('focus-in-event', lambda x, y: self.search_entry.grab_focus())

		search_box = gtk.HBox()
		search_box.pack_start(search_allow_button, gtk.FALSE, gtk.TRUE, 0)
		search_box.pack_start(self.search_entry)

		status_vbox = gtk.VBox(gtk.FALSE, 0)
		status_vbox.pack_start(self.ghost_name, gtk.FALSE, gtk.FALSE, 0)
		status_vbox.pack_start(v_paned, gtk.TRUE, gtk.TRUE, 5)
		status_vbox.pack_start(search_box, gtk.FALSE, gtk.FALSE, 0)

## Create Bottons (right) window
		ctrl_vbox = gtk.VBox(gtk.FALSE, 0)
		for item in [
			[gtk.STOCK_ADD,     unicode(_('Add'),    'utf-8'), self.add_ghost,    None],
			[gtk.STOCK_YES,     unicode(_('FMO'),    'utf-8'), self.check_fmo,    None],
			[gtk.STOCK_NO,      unicode(_('Use'),    'utf-8'), self.check_use,    None],
			[gtk.STOCK_REMOVE,  unicode(_('Remove'), 'utf-8'), self.delete_ghost, [ord('D'), gtk.gdk.CONTROL_MASK]],
			[gtk.STOCK_REFRESH, unicode(_('Reload'), 'utf-8'), self.reload,       [ord('R'), gtk.gdk.CONTROL_MASK]],
			[gtk.STOCK_SAVE,    unicode(_('Save'),   'utf-8'), self.save,         [ord('S'), gtk.gdk.CONTROL_MASK]],
			[gtk.STOCK_CLOSE,   unicode(_('Close'),  'utf-8'), self.close,        [ord('Q'), gtk.gdk.CONTROL_MASK]],
			]:
			btn = get_icon_button(item[0], size=gtk.ICON_SIZE_BUTTON, relief=gtk.RELIEF_NORMAL, label=item[1])
			btn.connect('clicked', item[2])

			if item[3] is not None:
				btn.add_accelerator('clicked', accel_group, item[3][0], item[3][1], gtk.ACCEL_VISIBLE)

			ctrl_vbox.pack_start(btn, gtk.TRUE, gtk.TRUE, 5)

		main_hbox = gtk.HBox(gtk.FALSE, 0)
		main_hbox.pack_start(status_vbox, gtk.TRUE, gtk.TRUE, 0)
		main_hbox.pack_start(ctrl_vbox, gtk.FALSE, gtk.FALSE, 5)

## Create Base Window
		gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
		self.connect("delete-event", self.close)
		self.set_title(unicode(_(APP), "utf-8") + ": " + unicode(_("SVGManager"), "utf-8"))
		self.set_default_size(450, 450)
		self.add_accel_group(accel_group)
		self.add(main_hbox)
		self.clear_info()

	def open(self, widget=None, data=None):
		self.show_all()
		#self.ghost_list.grab_focus()

	def close(self, widget=None, event=None, data=None):
		self.hide()
		return gtk.TRUE

	def browser_changed(self, browser):
		self.browse_command = browser

	def sort(self, treeviewcol, col_num):
		id = self.ghost_list_str.get_sort_column_id()
		if id[0] == col_num:
			self.ghost_list_str.set_sort_column_id(col_num, not id[1])
		else:
			self.ghost_list_str.set_sort_column_id(col_num, gtk.SORT_DESCENDING)

	def search(self, widget=None, data=None):
		pat = re.compile(self.search_entry.get_text(), re.IGNORECASE)
		current = self.selection()
		rng = []

		if self.s_arrow.get_property('arrow-type') == gtk.ARROW_UP:
			if current == None: current = len(self.ghost_list_str)
			rng = range(0, current)
			rng.reverse()
		elif self.s_arrow.get_property('arrow-type') == gtk.ARROW_DOWN:
			if current == None: current = 0
			rng = range(current+1, len(self.ghost_list_str))

		for i in rng:
			for j in range(1, 3):
				m = pat.search(self.ghost_list_str.get_value(self.ghost_list_str.get_iter(i), j))
				if m:
					self.ghost_list.set_cursor(i)
					return None

	def is_selected(self):
		try:
			treeselection = self.ghost_list.get_selection()
			[liststore, treeiter] = treeselection.get_selected()
			if not treeiter:
				return False
		except:
			return False
		return True

	def selection(self):
		if self.is_selected():
			treeselection = self.ghost_list.get_selection()
			[liststore, treeiter] = treeselection.get_selected()
			pathes = liststore.get_path(treeiter)
			return pathes[0]
		else:
			return None

	def entry_keypress(self, widget, event):
		return self.scroll(widget, event)

	def set_entry_guide(self, widget, event, data):
		if data == 'in':
			if widget.get_text() == self.msg:
				widget.set_text('')
		elif data == 'out':
			if not widget.get_text():
				widget.set_text(self.msg)

	def scroll(self, widget, event):
		if event.state == 1: # shift
			if event.keyval == 65362: # up
				self.sw.emit('scroll-child', gtk.SCROLL_PAGE_UP, gtk.FALSE)
				return gtk.TRUE
			elif event.keyval == 65364: # down
				self.sw.emit('scroll-child', gtk.SCROLL_PAGE_DOWN, gtk.FALSE)
				return gtk.TRUE
			return gtk.FALSE
		return gtk.FALSE

## 'cursor-changed' signal gtk.TreeView
	def cursor_changed(self, treeview):
		treeview.emit("select-cursor-row", gtk.FALSE)

	def set_info(self, ghost_name='', sakura='', unyu='', craftman='', hpname='', hpurl='', supported='',
				 action='', text_path='', surface_path='', version='', info=''):
		status = [
			['sakura',		sakura],
			['unyu',		unyu],
			['craftman',	craftman],
			['HPname',		hpname],
			['URL',			hpurl],
			['supported',	supported],
			['action',		action],
			['DEF.',		text_path],
			['SUF.',		surface_path],
			['Ver.',		version],
			['info',		info],
			]

		self.info_svg_str.clear()
		for l in status:
			self.info_svg_str.append(l)
		self.ghost_name.set_text(ghost_name)

	def clear_info(self):
		self.sur_button.set_sensitive(gtk.FALSE)
		self.url_button.set_sensitive(gtk.FALSE)
		self.sakura_box.set_from_pixbuf(gtk.gdk.pixbuf_new_from_xpm_data(XPM_VANISH))
		self.unyuu_box.set_from_pixbuf(gtk.gdk.pixbuf_new_from_xpm_data(XPM_VANISH))
		self.set_info()
		self.disconnect_buttons()

	def error_info(self, txt_path):
		self.sur_button.set_sensitive(gtk.FALSE)
		self.url_button.set_sensitive(gtk.FALSE)
		self.sakura_box.set_from_stock(gtk.STOCK_MISSING_IMAGE, gtk.ICON_SIZE_LARGE_TOOLBAR)
		self.unyuu_box.set_from_stock(gtk.STOCK_MISSING_IMAGE, gtk.ICON_SIZE_LARGE_TOOLBAR)
		self.set_info(unicode(_(txt_path), 'utf-8') + unicode(_(' is NOT existing!!'), 'utf-8'))
		self.disconnect_buttons()

	def disconnect_buttons(self):
		if self.button_id:
			self.url_button.disconnect(self.button_id)
		self.button_id = None

		if self.button_id_s:
			self.sur_button.disconnect(self.button_id_s)
		self.button_id_s = None

	def select_row(self, treeview, se=gtk.FALSE):
		if len(self.ghost_list_str) <= 0:
			self.clear_info()
			return gtk.TRUE

		txt_path = expand_quote(self.ghost_list_str.get_value(self.ghost_list_str.get_iter(self.ghost_list.get_cursor()[0][0]), 3))
		txt_path = os.path.join(self.ghostmanager.get_ghost_dir(), os.path.basename(txt_path))
		g = self.ghostmanager.nominate_ghost_txt(txt_path)
		if g is None:
			self.error_info(txt_path)
			#self.clear_info()
			#self.set_info()
			#self.info_svg_str.clear()
			#self.sur_button.set_sensitive(gtk.FALSE)
			#self.url_button.set_sensitive(gtk.FALSE)
			#self.ghost_name.set_text(unicode(_(txt_path), 'utf-8') + unicode(_(' is NOT existing!!'), 'utf-8'))
			#self.sakura_box.set_from_stock(gtk.STOCK_MISSING_IMAGE, gtk.ICON_SIZE_LARGE_TOOLBAR)
			#self.unyuu_box.set_from_stock(gtk.STOCK_MISSING_IMAGE, gtk.ICON_SIZE_LARGE_TOOLBAR)
			return gtk.TRUE

		## get and show sakura, unyuu surfaces
		sur_sakura = g.get_surface(0)
		sur_unyuu = g.get_surface(10)

		self.sakura_box.set_from_pixbuf(sur_sakura)
		self.unyuu_box.set_from_pixbuf(sur_unyuu)

		hpurl = g.get_url()
		#self.ghost_name.set_text(g.get_ghostname())

		self.set_info(g.get_ghostname(), g.get_sakura(), g.get_unyu(), g.get_craftman(),
					  g.get_hpname(), hpurl, g.get_supported(), g.get_sakura_action(), txt_path,
					  os.path.join(self.ghostmanager.get_ghost_dir(), g.get_surfacefile()),
					  g.get_update(), g.get_inform())

# 		status = [
# 			['sakura',		g.get_sakura()],
# 			['unyu',		g.get_unyu()],
# 			['craftman',	g.get_craftman()],
# 			['HPname',		g.get_hpname()],
# 			['URL',			hpurl],
# 			['supported',	g.get_supported()],
# 			['action',		g.get_sakura_action()],
# 			['DEF.',		txt_path],
# 			['SUF.',		os.path.join(self.ghostmanager.get_ghost_dir(), g.get_surfacefile())],
# 			['Ver.',		g.get_update()],
# 			['info',		g.get_inform()]
# 			]

# 		self.info_svg_str.clear()
		
# 		for i in range(len(status)):
# 			self.info_svg_str.append(status[i])

# 		if self.button_id:
# 			self.url_button.disconnect(self.button_id)
# 		self.button_id = None

# 		if self.button_id_s:
# 			self.sur_button.disconnect(self.button_id_s)
# 		self.button_id_s = None

		self.disconnect_buttons()

		if hpurl:
			self.url_button.set_sensitive(gtk.TRUE)
			self.button_id = self.url_button.connect("clicked", self.url_jump, hpurl)
		else:
			self.url_button.set_sensitive(gtk.FALSE)

		self.sur_button.set_sensitive(gtk.TRUE)
		self.button_id_s = self.sur_button.connect("clicked", self.show_surface, g)

	def _copy_svg(self, src_file, dest_file):
		if os.path.exists(src_file) and os.path.exists(dest_file) and \
			   os.path.samefile(src_file, dest_file):
			return True

		if os.path.exists(dest_file):
			d = get_simple_yes_or_no_dialog(unicode(_('Really?'), 'utf-8'),
											unicode(_('File already exists.'), 'utf-8') + \
											' ' + unicode(_('overwrite?'), 'utf-8') + \
											'\n%s' % dest_file, self)
			d.show()
			res = d.run()
			d.destroy()
			if not res or res != gtk.RESPONSE_YES:
				return False
		try:
			shutil.copyfile(src_file, dest_file)
		except:
			return False
		return True

	def copy_svg(self, src_textfile):
		src_textfile = os.path.abspath(src_textfile)
		ghost_dir    = self.ghostmanager.get_ghost_dir()

		if not os.path.exists(src_textfile):
			open_error_dialog(unicode(_('File not found'), 'utf-8') + \
							  ': %s' % src_textfile, self)
			return None
		src_dir, textfile = os.path.split(src_textfile)
		try:
			ghost = self.ghostmanager.nominate_ghost_txt(textfile, src_dir)
		except:
			open_error_dialog(unicode(_('Invalid file'), 'utf-8') + \
							  ': %s' % src_textfile, self)
			return None
		imagefile     = ghost.get_surfacefile()
		src_imagefile = os.path.join(src_dir, imagefile)
		if not os.path.exists(src_imagefile):
			open_error_dialog(unicode(_('File not found'), 'utf-8') + \
							  ': %s' % src_imagefile, self)
			return None

		dest_dir       = self.ghostmanager.get_ghost_dir()
		dest_textfile  = os.path.join(ghost_dir, textfile)
		dest_imagefile = os.path.join(ghost_dir, imagefile)

		if self._copy_svg(src_textfile, dest_textfile):
			if self._copy_svg(src_imagefile, dest_imagefile):
				return ghost
			return None
		return None

## if pressed button
	def add_ghost(self, button):
		ghost_dir = self.ghostmanager.get_ghost_dir()

		filew = gtk.FileSelection()
		filew.set_select_multiple(gtk.FALSE)
		filew.connect("destroy", lambda x: filew.destroy())
		filew.set_filename(os.path.normpath(os.path.join(ghost_dir, '*.txt')))
		filew.show()
		res = filew.run()
		src_svg_file = filew.get_filename()
		filew.destroy()

		if res != gtk.RESPONSE_OK:
			return gtk.TRUE

		ghost = self.copy_svg(src_svg_file)
		if not ghost:
			sys.stderr.write("!!! couldn't copy %s\n" % src_svg_file)
			return gtk.TRUE

		ghost_name = ghost.get_sakura2()
		if not ghost_name:
			ghost_name = ghost.get_sakura()

		if not ghost_name and not ghost.get_ghostname():
			open_error_dialog(unicode(_("Couldn't open file"), 'utf-8'), self)
			return gtk.TRUE

		self.ghost_list_str.append([gtk.TRUE, ghost_name, '',
							'"%s"' % os.path.join(os.path.basename(ghost_dir),
												  os.path.basename(src_svg_file))])
		self.ghost_list.set_cursor(len(self.ghost_list_str)-1)
		return gtk.TRUE

	def check_fmo(self, button):
		try:
			i = self.ghost_list.get_cursor()[0][0]
			now = self.ghost_list.get_selection()
			[model, iter] = now.get_selected()
			if model.get_value(iter, 2) == 'FMO':
				self.delete_fmo(model, iter)
			else:
				self.set_fmo(model, iter)
			self.ghost_list.set_cursor(i)
			#self.ghost_list.grab_focus()
		except:
			open_error_dialog(unicode(_('No selected Ghost!'), 'utf-8'), self)

	def check_use(self, button):
		try:
			i = self.ghost_list.get_cursor()[0][0]
			now = self.ghost_list.get_selection()
			[model, iter] = now.get_selected()
			if model.get_value(iter, 0):
				self.unuse_ghost(model, iter)
			else:
				self.use_ghost(model, iter)
			self.ghost_list.set_cursor(i)
			#self.ghost_list.grab_focus()
		except:
			open_error_dialog(unicode(_('No selected Ghost!'), 'utf-8'), self)

	def delete_ghost(self, button):
		try:
			i = self.ghost_list.get_cursor()[0][0]
			self.ghost_list_str.remove(self.ghost_list_str.get_iter(i))
			if i == len(self.ghost_list_str):
				self.ghost_list.set_cursor(i-1)
			else:
				self.ghost_list.set_cursor(i)
		except:
			open_error_dialog(unicode(_('No selected Ghost!'), 'utf-8'), self)

	def reload(self, button):
		self.get_config()
		self.app.notify_svg_changed()
		#self.ghost_list.grab_focus()

	def save(self, button):
		self.set_config()
		self.app.notify_svg_changed()

	def get_ghost(self, ifghost):
		return self.ghostmanager.nominate_ghost(ifghost)

	def get_fmo_ghostnames(self):
		return self.ghostmanager.get_fmo_ghostnames()

	def check_use_toggle(self, cellrenderer, path):
		model = self.ghost_list_str
		iter = model.get_iter(path)
		try:
			if self.ghost_list_str.get_value(iter, 0):
				self.unuse_ghost(model, iter)
			else:
				self.use_ghost(model, iter)
		except:
			open_error_dialog(unicode(_('No selected Ghost!'), 'utf-8'), self)

	def url_jump(self, button, hpurl):
		try:
			os.system(self.browse_command % hpurl + ' &')
		except:
			open_error_dialog(unicode(_('No selected Ghost!'), 'utf-8'), self)

	def show_surface(self, button, ghost):
		path = os.path.join(self.ghostmanager.get_ghost_dir(), ghost.get_surfacefile())
		try:
			pixbuf = gtk.gdk.pixbuf_new_from_file(path)
		except:
			return
		ss = ShowingSurface(self.app, pixbuf, ghost)

	def get_config(self):
		self.ghost_list_str.clear()
		self.ghostmanager.generate_dictionary()
		for line in self.ghostmanager.get_dict().values():
			self.ghost_list_str.append(line)

	def set_config(self):
		self.ghostmanager.save_dictionary(self.ghost_list_str)

	def use_ghost(self, model, iter):
		## delete '#' from  first item '#GHOST'
		self.ghost_list_str.set_value(iter, 0, gtk.TRUE)

	def unuse_ghost(self, model, iter):
		## insert '#' in first item 'GHOST'
		self.ghost_list_str.set_value(iter, 0, gtk.FALSE)

	def set_fmo(self, model, iter):
		## insert 'FMO' the third item
		self.ghost_list_str.set_value(iter, 2, 'FMO')

	def delete_fmo(self, model, iter):
		## delete 'FMO' the third item 'FMO'
		self.ghost_list_str.set_value(iter, 2, '')

if __name__ == "__main__":
	sm = SVGManagerWindow()
	gtk.main()
